__version__ = "0.0.1"
AUTHOR = "Vanessa Sochat"
AUTHOR_EMAIL = "vsoch@users.noreply.github.com"
NAME = "usernetes"
PACKAGE_URL = "https://github.com/converged-computing/usernetes-python"
KEYWORDS = "cluster, orchestration, user-space kubernetes, kubernetes, compose"
DESCRIPTION = "Python wrapper for user-space Kubernetes 'usernetes'"
LICENSE = "LICENSE"

################################################################################
# Global requirements

INSTALL_REQUIRES = []
TESTS_REQUIRES = (("pytest", {"min_version": "4.6.2"}),)
INSTALL_REQUIRES_ALL = TESTS_REQUIRES
